/* $Id: marshal.c,v 1.8 2004/05/13 17:48:59 mikpe Exp $
 * Performance-monitoring counters driver.
 * Structure marshalling support, kernel-level code.
 *
 * Copyright (C) 2003-2004  Mikael Pettersson
 */
#include <linux/config.h>
struct inode;
#include <linux/sched.h>
#include <linux/perfctr.h>
#include <linux/errno.h>
#include <linux/stddef.h>
#include <linux/string.h>
#include <asm/uaccess.h>

#include "marshal.h"
#include "marshal_core.c"

int perfctr_copy_from_user(void *struct_address,
			   struct perfctr_struct_buf *argp,
			   const struct perfctr_struct_desc *sdesc)
{
	struct perfctr_marshal_stream stream;

	if( get_user(stream.size, &argp->rdsize) )
		return -EFAULT;
	stream.buffer = argp->buffer;
	stream.pos = 0;
	stream.error = 0;
	memset(struct_address, 0, sdesc->total_sizeof);
	return perfctr_decode_struct(struct_address, sdesc, &stream);
}

int perfctr_copy_to_user(struct perfctr_struct_buf *argp,
			 void *struct_address,
			 const struct perfctr_struct_desc *sdesc)
{
	struct perfctr_marshal_stream stream;

	if( get_user(stream.size, &argp->wrsize) )
		return -EFAULT;
	stream.buffer = argp->buffer;
	stream.pos = 0;
	stream.error = 0;
	perfctr_encode_struct(struct_address, sdesc, &stream);
	if( stream.error )
		return stream.error;
	if( put_user(stream.pos, &argp->rdsize) )
		return -EFAULT;
	return 0;
}
