/* $Id: marshal.c,v 1.2 2004/05/13 23:35:27 mikpe Exp $
 * Performance-monitoring counters driver.
 * Structure marshalling support, user-level code.
 *
 * Copyright (C) 2003-2004  Mikael Pettersson
 */
#include "libperfctr.h"
#include <errno.h>
#include <stddef.h>
#include <string.h>

#include "marshal.h"
#define put_user(w, p) (*(p) = (w), 0)
#define get_user(w, p) ((w) = *(p), 0)
#define CONFIG_PERFCTR_VIRTUAL
#define CONFIG_PERFCTR_GLOBAL
#include "marshal_core.c"

#define sdesc_bufsize(sdesc)	((sdesc)->total_nrfields + (sdesc)->total_sizeof/sizeof(int))

static int common_sys_w(const void *arg,
			const struct perfctr_struct_desc *sdesc,
			struct perfctr_struct_buf *buf,
			unsigned int bufsize)
{
	struct perfctr_marshal_stream stream;

	stream.size = bufsize;
	stream.buffer = buf->buffer;
	stream.pos = 0;
	stream.error = 0;
	perfctr_encode_struct(arg, sdesc, &stream);
	if( stream.error ) {
		errno = -stream.error;
		return -1;
	}
	buf->rdsize = stream.pos;
	return 0;
}

int perfctr_sys_w(int fd, unsigned int cmd, const void *arg,
		  const struct perfctr_struct_desc *sdesc)
{
	unsigned int bufsize = sdesc_bufsize(sdesc);
	union {
		struct perfctr_struct_buf buf;
		struct {
			unsigned int rdsize;
			unsigned int wrsize;
			unsigned int buffer[bufsize];
		} buf_bufsize;
	} u;
	int err;

	err = common_sys_w(arg, sdesc, &u.buf, bufsize);
	if( err < 0 )
		return err;
	u.buf.wrsize = 0;
	return _sys_perfctr(cmd, fd, &u.buf);
}

static int common_sys_r(int fd, unsigned int cmd, void *res,
			const struct perfctr_struct_desc *sdesc,
			struct perfctr_struct_buf *buf)
{
	struct perfctr_marshal_stream stream;
	int err;

	if( _sys_perfctr(cmd, fd, buf) < 0 )
		return -1;
	stream.size = buf->rdsize;
	stream.buffer = buf->buffer;
	stream.pos = 0;
	stream.error = 0;
	memset(res, 0, sdesc->total_sizeof);
	err = perfctr_decode_struct(res, sdesc, &stream);
	if( err < 0 ) {
		errno = -err;
		return -1;
	}
	return 0;
}

int perfctr_sys_r(int fd, unsigned int cmd, void *res,
		  const struct perfctr_struct_desc *sdesc)
{
	unsigned int bufsize = sdesc_bufsize(sdesc);
	union {
		struct perfctr_struct_buf buf;
		struct {
			unsigned int rdsize;
			unsigned int wrsize;
			unsigned int buffer[bufsize];
		} buf_bufsize;
	} u;

	u.buf.rdsize = 0;
	u.buf.wrsize = bufsize;
	return common_sys_r(fd, cmd, res, sdesc, &u.buf);
}

int perfctr_sys_wr(int fd, unsigned int cmd, void *argres,
		   const struct perfctr_struct_desc *arg_sdesc,
		   const struct perfctr_struct_desc *res_sdesc)
{
	unsigned int arg_bufsize = sdesc_bufsize(arg_sdesc);
	unsigned int res_bufsize = sdesc_bufsize(res_sdesc);
	unsigned int bufsize = arg_bufsize > res_bufsize ? arg_bufsize : res_bufsize;
	union {
		struct perfctr_struct_buf buf;
		struct {
			unsigned int rdsize;
			unsigned int wrsize;
			unsigned int buffer[bufsize];
		} buf_bufsize;
	} u;
	int err;

	err = common_sys_w(argres, arg_sdesc, &u.buf, arg_bufsize);
	if( err < 0 )
		return err;
	u.buf.wrsize = res_bufsize;
	return common_sys_r(fd, cmd, argres, res_sdesc, &u.buf);
}
